#!/usr/bin/env bash

# backup.sh - functions for backup processing.
# Copyright (c) 2022 ge <https://nixhacks.net/>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

process_source() {
    # Run handler function for source URI by scheme.
    #
    # Usage: process_source URI

    local uri
    local scheme
    local handler

    uri="$1"
    scheme="${uri%%:*}"
    # Process Path-only URI
    if [[ "$scheme" == "$uri" ]]; then scheme='file'; fi

    echo -e "Processing source $uri ..." |
        sed 's/:\/\//\[auth\]/;s/:.*@/:xxx@/;s/\[auth\]/:\/\//'
        # ^^^ hide password from output ^^^

    case "$scheme" in
        file)           handler='src_tar';;
        mysql|mariadb)  handler='src_mysqldump';;
        postgres)       handler='src_pg_dump';;
        sqlite)         handler='src_sqlite';;
        *)  # shellcheck disable=SC2154
            # 'backup_script' is assigned in main script.
            echo "Error: $backup_script: Unsupported URI scheme: $scheme" >&2
            exit 1;;
    esac

    # Run handler function
    "$handler" "$uri"
}

process_target() {
    # Run handler function for target URI by scheme.
    #
    # Usage: process_target URI

    local uri
    local scheme
    local handler

    uri="$1"
    scheme="${uri%%:*}"
    # Process Path-only URI
    if [[ "$scheme" == "$uri" ]]; then scheme='file'; fi

    echo -e "Processing target $uri ..."

    case "$scheme" in
        file)   handler='tgt_cp';;
        ftp)    handler='tgt_ftp';;
        sftp)   handler='tgt_sftp';;
        rsync)  handler='tgt_rsync';;
        s3)     handler='tgt_s3cmd';;
        sj)     handler='tgt_sj';;
        swift)  handler='tgt_swift';;
        dav)    handler='tgt_dav';;
        davs)   handler='tgt_davs';;
        *) echo "Error: Unsupported URI scheme: $scheme" >&2; exit 1;;
    esac

    # Run handler function
    "$handler" "$uri"
}

builtin_backup() {
    # Backup function.
    #
    # Usage: builtin_backup

    # shellcheck disable=SC2154
    for source in "${sources[@]}"; do
        process_source "$source"
    done

    # shellcheck disable=SC2154
    for target in "${targets[@]}"; do
        process_target "$target"
    done
}
